CheckBlockError

Enum CheckBlockError 

pub enum CheckBlockError<N: Network> {
    BlockAlreadyExists {
        hash: N::BlockHash,
    },
    InvalidHeight {
        expected: u32,
        actual: u32,
    },
    InvalidHash,
    InvalidPrefix {
        index: usize,
        error: Box<CheckBlockError<N>>,
    },
    SolutionAlreadyExists {
        solution_id: SolutionID<N>,
    },
    SpeculationFailed {
        inner: Error,
    },
    VerificationFailed {
        inner: Error,
    },
    SolutionLimitReached {
        prover_address: Address<N>,
    },
    PreviousSolutionNotFound {
        solution_id: SolutionID<N>,
    },
    PreviousTransactionNotFound {
        transaction_id: N::TransactionID,
    },
    Other(Error),
}
Error returned by [Self::check_block_subdag] and [Self::check_block_subdag_inner].

This allows parsing for begning errors, such as the block already existing in the ledger.

Variants



BlockAlreadyExists

Fields

hash: N::BlockHash


InvalidHeight

Fields

expected: u32
actual: u32


InvalidHash



InvalidPrefix

An error related to the given prefix of pending blocks.

Fields

index: usize


SolutionAlreadyExists

Fields

solution_id: SolutionID<N>


SpeculationFailed

Fields

inner: Error


VerificationFailed

Fields

inner: Error


SolutionLimitReached

Fields

prover_address: Address<N>


PreviousSolutionNotFound

Fields

solution_id: SolutionID<N>


PreviousTransactionNotFound

Fields

transaction_id: N::TransactionID


Other(Error)

Implementations