pub enum DeploymentVersion {
V1 = 1,
V2 = 2,
V3 = 3,
}Expand description
The deployment version.
Variants
V1 = 1
A deployment without a program checksum or program owner (V1). Inactive after consensus version >= V9.
V2 = 2
A deployment with both a program checksum and program owner (V2). Active after consensus version >= V9.
V3 = 3
A deployment with a program checksum but without a program owner (V3). Active after consensus version >= V14. This is used for amendments.