pub enum Output<N>where
N: Network,{
Constant(Field<N>, Option<Plaintext<N>>),
Public(Field<N>, Option<Plaintext<N>>),
Private(Field<N>, Option<Ciphertext<N>>),
Record(Field<N>, Field<N>, Option<Record<N, Ciphertext<N>>>, Option<Field<N>>),
ExternalRecord(Field<N>),
Future(Field<N>, Option<Future<N>>),
DynamicRecord(Field<N>),
RecordWithDynamicID(Field<N>, Field<N>, Option<Record<N, Ciphertext<N>>>, Option<Field<N>>, Field<N>),
ExternalRecordWithDynamicID(Field<N>, Field<N>),
}Expand description
The transition output.
Variants
Constant(Field<N>, Option<Plaintext<N>>)
The plaintext hash and (optional) plaintext.
Public(Field<N>, Option<Plaintext<N>>)
The plaintext hash and (optional) plaintext.
Private(Field<N>, Option<Ciphertext<N>>)
The ciphertext hash and (optional) ciphertext.
Record(Field<N>, Field<N>, Option<Record<N, Ciphertext<N>>>, Option<Field<N>>)
The commitment, checksum, (optional) record ciphertext, and (optional) sender ciphertext.
ExternalRecord(Field<N>)
The hash of the external record’s (function_id, record, tvk, output index).
Future(Field<N>, Option<Future<N>>)
The future hash and (optional) future.
DynamicRecord(Field<N>)
The hash of the dynamic record’s (function_id, record, tvk, output index).
RecordWithDynamicID(Field<N>, Field<N>, Option<Record<N, Ciphertext<N>>>, Option<Field<N>>, Field<N>)
The commitment, checksum, (optional) record ciphertext, (optional) sender ciphertext, and dynamic ID.
ExternalRecordWithDynamicID(Field<N>, Field<N>)
The external record hash and dynamic ID.
Implementations
Sourceimpl<N> Output<N>where
N: Network,
impl<N> Output<N>where
N: Network,
Sourcepub fn to_transition_leaf(&self, index: u8) -> TransitionLeaf<N>
pub fn to_transition_leaf(&self, index: u8) -> TransitionLeaf<N>
Returns the output as a transition leaf. Note: RecordWithDynamicID uses leaf variant 3 (same as Record) with version 2. Note: ExternalRecordWithDynamicID uses leaf variant 4 (same as ExternalRecord) with version 2.
Sourcepub const fn record(&self) -> Option<(&Field<N>, &Record<N, Ciphertext<N>>)>
pub const fn record(&self) -> Option<(&Field<N>, &Record<N, Ciphertext<N>>)>
Returns the commitment and record, if the output is a record.
Sourcepub fn into_record(self) -> Option<(Field<N>, Record<N, Ciphertext<N>>)>
pub fn into_record(self) -> Option<(Field<N>, Record<N, Ciphertext<N>>)>
Consumes self and returns the commitment and record, if the output is a record.
Sourcepub const fn commitment(&self) -> Option<&Field<N>>
pub const fn commitment(&self) -> Option<&Field<N>>
Returns the commitment, if the output is a record.
Sourcepub fn into_commitment(self) -> Option<Field<N>>
pub fn into_commitment(self) -> Option<Field<N>>
Returns the commitment, if the output is a record, and consumes self.
Sourcepub fn into_nonce(self) -> Option<Group<N>>
pub fn into_nonce(self) -> Option<Group<N>>
Returns the nonce, if the output is a record, and consumes self.
Sourcepub const fn checksum(&self) -> Option<&Field<N>>
pub const fn checksum(&self) -> Option<&Field<N>>
Returns the checksum, if the output is a record.
Sourcepub fn into_checksum(self) -> Option<Field<N>>
pub fn into_checksum(self) -> Option<Field<N>>
Returns the checksum, if the output is a record, and consumes self.
Sourcepub const fn sender_ciphertext(&self) -> Option<&Field<N>>
pub const fn sender_ciphertext(&self) -> Option<&Field<N>>
Returns the sender ciphertext, if the output is a record.
Sourcepub fn into_sender_ciphertext(self) -> Option<Field<N>>
pub fn into_sender_ciphertext(self) -> Option<Field<N>>
Returns the sender ciphertext, if the output is a record, and consumes self.
Sourceimpl<N> Output<N>where
N: Network,
impl<N> Output<N>where
N: Network,
Sourcepub fn decrypt_sender_ciphertext(
&self,
account_view_key: &ViewKey<N>,
) -> Result<Option<Address<N>>, Error>
pub fn decrypt_sender_ciphertext( &self, account_view_key: &ViewKey<N>, ) -> Result<Option<Address<N>>, Error>
Returns the sender address, given the account view key of the record owner.
If the output is not a record or does not contain a sender ciphertext, it returns Ok(None).
If the record does not belong to the given account view key, it returns Err.
If the sender ciphertext is malformed or cannot be decrypted, it returns Err.
Sourceimpl<N> Output<N>where
N: Network,
impl<N> Output<N>where
N: Network,
Sourcepub fn verifier_inputs(&self) -> impl Iterator<Item = <N as Environment>::Field>
pub fn verifier_inputs(&self) -> impl Iterator<Item = <N as Environment>::Field>
Returns the public verifier inputs for the proof.
Sourcepub const fn dynamic_id(&self) -> Option<&Field<N>>
pub const fn dynamic_id(&self) -> Option<&Field<N>>
Returns the dynamic ID, if the output carries one.
Sourcepub fn to_caller_output(&self) -> Output<N>
pub fn to_caller_output(&self) -> Output<N>
Returns the output from the caller’s perspective.
Sourcepub fn verify(
&self,
function_id: Field<N>,
tcm: &Field<N>,
index: usize,
) -> bool
pub fn verify( &self, function_id: Field<N>, tcm: &Field<N>, index: usize, ) -> bool
Returns true if the output is well-formed.
If the optional value exists, this method checks that it hashes to the output ID.
Sourcepub fn is_type(&self, expected_value_type: &ValueType<N>) -> bool
pub fn is_type(&self, expected_value_type: &ValueType<N>) -> bool
Returns true if the output matches the expected value type.
Sourcepub fn valid_as_output(
&self,
callee_value_type: &ValueType<N>,
is_dynamic: Option<bool>,
) -> bool
pub fn valid_as_output( &self, callee_value_type: &ValueType<N>, is_dynamic: Option<bool>, ) -> bool
Returns true if the output matches the given value type as produced by the callee.
Record, ExternalRecord, DynamicRecord, RecordWithDynamicID and
ExternalRecordWithDynamicID outputs are handled according to the value of is_dynamic:
Some(true)enforces type correspondence assuming the outputs refer to a dynamic call.Some(false)enforces it assuming a static call.Noneis flexible and makes no assumption about the type of call: both dynamic-call and static-call matching patterns returntrue.
For outputs of an invalid type according to is_dynamic, this function returns false
regardless of the value type.