pub enum CheckBlockError<N: Network> {
BlockAlreadyExists {
hash: N::BlockHash,
},
InvalidHeight {
expected: u32,
actual: u32,
},
InvalidHash,
InvalidPrefix {
index: usize,
error: Box<CheckBlockError<N>>,
},
SolutionAlreadyExists {
solution_id: SolutionID<N>,
},
SpeculationFailed {
inner: Error,
},
VerificationFailed {
inner: Error,
},
SolutionLimitReached {
prover_address: Address<N>,
},
PreviousSolutionNotFound {
solution_id: SolutionID<N>,
},
PreviousTransactionNotFound {
transaction_id: N::TransactionID,
},
Other(Error),
}Expand description
Error returned by [Self::check_block_subdag] and [Self::check_block_subdag_inner].
This allows parsing for begning errors, such as the block already existing in the ledger.
Variants
BlockAlreadyExists
InvalidHeight
InvalidHash
InvalidPrefix
An error related to the given prefix of pending blocks.
SolutionAlreadyExists
Fields

solution_id: SolutionID<N>SpeculationFailed
VerificationFailed
SolutionLimitReached
PreviousSolutionNotFound
Fields

solution_id: SolutionID<N>PreviousTransactionNotFound
Fields

transaction_id: N::TransactionIDOther(Error)
Implementations
Sourceimpl<N: Network> CheckBlockError<N>
impl<N: Network> CheckBlockError<N>
pub fn into_anyhow(self) -> Error